/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// AOLSession.h

#ifndef __AOL_SESSION__
#define __AOL_SESSION__

#include "XPlatform.h"
#include "XErrors.h"
#include "P3.h"
#include "Modules.h"

// ** Types
typedef enum SessionStatus {
	SS_Disconnected,
	SS_Connected,
	SS_SigningOn,
	SS_Online
} SessionStatus;
	
typedef struct AOLSession {
	// Private Data
	int checkValue;
	void* connectionData;
	
	// Sign On Information
	char* screenname;
	char* password;
	
	// Session Information
	SessionStatus status;
	
	unsigned char arch;
	unsigned char versionMajor;
	unsigned char versionMinor;
	
	int P3Seq;
	int P3Ack;
	int heartBeat;
} AOLSession;

// ** Functions

// Portable Common Code
XAOLError	ASNew						(AOLSession** session);
void	  	ASDispose					(AOLSession* session);
int 		ASIsValid 					(AOLSession* session);
XAOLError	ASSendP3					(AOLSession* session, P3* p3);
XAOLError 	ASReceiveP3					(AOLSession* session, P3* p3);
XAOLError 	ASSignOn					(AOLSession* session, char* screenname, char* password);
XAOLError 	ASSignOff					(AOLSession* session);
XAOLError 	ASSetConnectionLayer		(Module* module);
XAOLError 	ASSetConnectionLayerByName	(char* connectionLayer);

// Implementation dependent definitions
XAOLError 	ASInit						();
XAOLError 	ASShutdown					();
XAOLError 	ASConnect					(AOLSession* session, void* connectionData);
XAOLError 	ASDisconnect				(AOLSession* session);
XAOLError 	ASSend						(AOLSession* session, char* buffer, int len);
XAOLError 	ASReceive					(AOLSession* session, char* buffer, int len, int* read);
void	  	ASSessionKill				(AOLSession* session);
int		  	ASGetError					();
void		ASIdle						();
	
#endif __AOL_SESSION__