/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// AOLStream.c

#include <string.h>

#include "AOLStream.h"
#include "Tokens.h"

typedef struct StreamListItem {
	AOLSession* session;
	char token[2];
	StreamHandlerPtr handler;
	void* data;
	struct StreamListItem* next;
} StreamListItem;

StreamListItem* streamList = NULL;
StreamListItem* streamListTail = NULL;
int numStreams = 0;

XAOLError AStreamAdd (AOLSession* session, char* token, StreamHandlerPtr handler, void* data) {	
	StreamListItem* item = NULL;
	
	if (session == NULL || strlen(token) > 2 || token == NULL || handler == NULL)
		return (Arg_Error);
	
	numStreams++;
	
	item = MAlloc(sizeof(StreamListItem));
	if (item == NULL)
		return (Mem_Error);
	
	item->session = session;
	item->handler = handler;
	item->data = data;
	strncpy(item->token, token, 2);
	item->next = NULL;
	
	if (streamList == NULL)
		streamList = item;
	else 
		streamListTail->next = item;
	
	streamListTail = item;
	return (No_Error);
	}
	
XAOLError AStreamRemove	(AOLSession* session, char* token, StreamHandlerPtr handler) {
	StreamListItem* leader, *trailer;
	
	if (numStreams <= 0 || streamList == NULL)
		return (Arg_Error);
		
	leader = streamList;
	
	while (leader != NULL) {
		if (leader->session == session &&
			strncmp(leader->token, token, 2) == 0 &&
			leader->handler == handler)
			break;
			
		trailer = leader;
		leader = leader->next;
		}
	
	if (leader == NULL)
		return (LItemNotFound);
	
	if (leader == streamList) {
		streamList = streamList->next;
		MFree((void*) leader);
		}
	else {
		trailer->next = leader->next;
		if (trailer->next == NULL)
			streamListTail = trailer;
		MFree((void*) leader);
		}
	
	return (No_Error);
	}
	
XAOLError AStreamDispatch (AOLSession* session, P3* p3) {
	StreamListItem* i = streamList;
	char token[2];
	char data[1024];
	XAOLError error;
	
	if (session == NULL || p3 == NULL)
		return (Arg_Error);
	
	if (i == NULL)
		return (No_Error);
		
	error = GetToken(p3, token, data);
	if (error != No_Error)
		return (error);
		
	// For right now we allow multiple registrations
	// May change
	// SBW - 2-4-2000
	while (i != NULL) {
		printf("Token asked for is %s and token read is %s.\n", i->token, token);
		if (i->session == session && strncmp(token, i->token, 2) == 0)
			i->handler(session, p3, i->data);
		
		i = i->next;
		}

	return (No_Error);
	}