/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// Events.c

#include "Events.h"
#include <stdio.h>

typedef struct ParserListItem {
	XAEventParser parser;
	struct ParserListItem* next;
	struct ParserListItem* previous;
} ParserListItem;

ParserListItem* parsers = NULL;

XAOLError XAERegister (XAEventParser parser) {
	ParserListItem* item = NULL;
	ParserListItem* end = parsers;
	
	printf("[Events] Registering New Parser.\n");
	item = MAlloc(sizeof(ParserListItem));
	if (item == NULL)
		return (Mem_Error);

	printf("Got new parser...\n");
	item->parser = parser;
	item->next = NULL;
	
	if (end == NULL) {
		printf("Adding first parser\n");
		item->previous = NULL;
		parsers = item;
		}
	else {
		printf("Added additional parser...\n");
		while (end->next != NULL)
			end = end->next;

		end->next = item;
		item->previous = end;
		}

	printf("[Events] Added new parser.\n");
	
	return (No_Error);
	}
	
XAOLError XAEUnregister (XAEventParser parser) {
	ParserListItem* item = parsers, *p, *n;

	while(item->parser != parser && item != NULL)
		item = item->next;

	if (item == NULL)
		return (LItemNotFound);

	p = item->previous;
	n = item->next;
	item->previous = NULL;
	item->next = NULL;
	p->next = n;
	MFree((void*) item);
	
	return (No_Error);
	}

void XAEDispatch (EventType what, AOLSession* where, EventData data) {
	XAOLEvent event;
	ParserListItem* i = parsers;
	
	if (parsers == NULL) {
		printf("[Events] No Parsers Loaded!\n");
		return;
		}
		
	event.what = what;
	event.where = where;
	event.data = data;
	
	while (i != NULL) {
		printf("[Events] Calling the parser!\n");
		i->parser(event);
		i = i->next;
		}
	}