/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// test.c

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include <Threads.h>

#include "test.h"

#include "ConnectionTypes.h"
#include "AOLSession.h"
#include "Tokens.h"
#include "Modules.h"
#include "P3.h"
#include "AOLProtocol.h"

#include "DCon.h"

void TokenTest();

AOLSession* session = NULL;
Module* mod = NULL;
Module* module = NULL;

void StartTest () {
	XAOLError error;
	
	Debug_Init();
	Debug_SetLevel(1);
	
	// Module Debugging - This ugliness *will* disappear
	XAOL_DEBUG_Print_("\n\nStarting Modular Test...\n");	
	
	error = ASInit();
	if (error != No_Error && error != 18) {
		XAOL_DEBUG_Print_("asinit failed!\n");
		XAOL_DEBUG_Printf_("Error that killed asinit %i\n", error);
		return;
		}
	else {
		XAOL_DEBUG_Print_("Loading Croall P3\n");
		if (module == NULL) {
			error = MLoad(&module, "Croall P3");
			if (error != No_Error) {
				if (error == ME_InternalError)
					XAOL_DEBUG_Printf_("Internal error code %i\n", MGetError());
				MUnload(module);
				return;
				}
		
			error = APSetProtocol(module);
			if (error != No_Error) {
				XAOL_DEBUG_Print_("APSetProtocol failed!\n");
				XAOL_DEBUG_Printf_("Error that killed APSetProtocol %i\n", error);
				MUnload(module);
				return;
				}
			}
		
		XAOL_DEBUG_Print_("Loading MacTCPConnection PPC!\n");
		if (mod == NULL) {
			error = MLoad(&mod, "MacTCPConnection PPC Debug");
			if (error != No_Error) {
				if (error == ME_InternalError)
					XAOL_DEBUG_Printf_("Internal error code %i\n", MGetError());
				MUnload(mod);
				return;
				}
		
			error = ASSetConnectionLayer(mod);
			if (error != No_Error) {
				XAOL_DEBUG_Print_("ASSetConnectionLayer failed!\n");
				XAOL_DEBUG_Printf_("Error that killed ASSetConnectionLayer %i\n", error);
				MUnload(mod);
				return;
				}
			}
			
		error = ASInit();
		if (error != No_Error) {
			XAOL_DEBUG_Print_("asinit failed!\n");
			XAOL_DEBUG_Printf_("Error that killed asinit %i\n", error);
			return;
			}
		}
	}

void Idle () {
	ASIdle();
	}
	
void AOLTest (char* sn, char* password) {
	XAOLError error;	
	TCPConnectionData tcpd;
	char* host = "americaonline.aol.com";
	unsigned char initPacket[] = { 0x00, 0x00 };
		
	tcpd.port = 5190;
	tcpd.hostname = (char*) malloc(strlen(host));
	snprintf(tcpd.hostname, strlen(host) + 1, "%s", host);
	
	SignOff();		
	
	error = ASNew(&session);
	if (session == NULL || error != No_Error) {
		XAOL_DEBUG_Printf_("asnew failed! (%i)\n", error);
		return;
		}
		
	error = ASConnect (session, (void*) &tcpd);
	if (error != No_Error) {
		XAOL_DEBUG_Printf_("asconnect failed! (%i)\n", error);
		ASDispose(session);
		return;
		}
	
	error = ASSignOn (session, sn, password);
	if (error != No_Error) {
		XAOL_DEBUG_Print_("assignon failed!\n");
		ASDispose(session);
		return;
		}
	
	error = PutToken (session, "SC", NULL, 0);
	if (error != No_Error)
		XAOL_DEBUG_Print_("PutToken failed!\n");
	
	error = PutToken (session, "m3", NULL, 0);
	if (error != No_Error)
		XAOL_DEBUG_Print_("PutToken failed!\n");
	
	TokenTest();
	}

void StopTest () {
	XAOLError error;
	
	SignOff();
		
	error = ASShutdown();
	}

void SignOff () {
	XAOLError error;
	
	if (session != NULL) {
		error = ASDisconnect (session);
		if (error != No_Error)
			XAOL_DEBUG_Printf_("asdisconnect failed!(%i)\n", error);
		ASDispose(session);
		session = NULL;
		}
	
	}

void TokenTest () {
	P3 * p3;
	unsigned char unknown1[] = { 121, 97 };
	XAOLError err;
	
	p3 = P3New(P3_TYPE_DATA);
	err = P3Append(p3, unknown1, sizeof(unknown1));
	err = ASSendP3(session, p3);
	XAOL_DEBUG_Printf_("WORD(%i)\n", err);
	P3Dispose(&p3);
	}