/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// test_client.c

// ** Headers
#include <Events.h>
#include <Threads.h>
#include <MacMemory.h>
#include <Menus.h>
#include <Fonts.h>
#include <MacWindows.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <Sound.h>
#include <Devices.h>
#include <Gestalt.h>
#include <Appearance.h>
#include <TextUtils.h>

#include <SIOUX.h>
#include "DCon.h"

#include "ThreadedConnections.h"

#include "test.h"

#include <stdio.h>

// ** Constants
enum {
	mMenuBar = 128,
	mApple = 128,
	mFile = 129
};

enum {
	iAbout = 1,
	iClient = 1,
	iSignOff = 2,
	iReload = 4,
	iQuit = 6
};

#define rAbout 128
#define rClient	129

// ** Globals
Boolean gQuit = false;
Boolean gHasAppearance = false;

// ** Prototypes
void 					EventLoop 			();
void 					ProcessEvents 		(EventRecord* event);
void 					HandleKeyPress		(EventRecord *event);
void 					HandleMouseDown		(EventRecord *event);
void 					HandleMenuCommand 	(long menuStuff);
Boolean 				ModalFilter 		(DialogPtr theDialog, EventRecord *theEvent, DialogItemIndex *itemHit);
void 					XAOLLogin 			();

void main () {	
	Handle menuBar;
	OSErr err;
	long result;
	
	// Initialize the toolbox
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
	
	// Check for Appearance and Register
	err = Gestalt(gestaltAppearanceAttr, &result);
	gHasAppearance = !err;
	
	if (gHasAppearance) RegisterAppearanceClient();
	
	// Set up the menubar
	menuBar = GetNewMBar(mMenuBar);
	if (menuBar == nil)
		ExitToShell();
	
	SetMenuBar(menuBar);
	
	DisposeHandle(menuBar);
	AppendResMenu(GetMenuHandle(mApple), 'DRVR');
		
	DrawMenuBar();
	
	StartTest();
	XAOLLogin();
	EventLoop();
	
	StopTest();
	}

// Your basic event loop, only handling a minimal set of events for this trivial sample
void EventLoop () {
	Boolean gotEvent;
	EventRecord event;
	
	while(! gQuit) {
		gotEvent = WaitNextEvent(everyEvent, &event, 35, nil);
		if (gotEvent) {
			if (! SIOUXHandleOneEvent(&event))
				ProcessEvents(&event);
			}
		else
			Idle();
		}
	}

// Process the events
void ProcessEvents (EventRecord* event) {
	switch (event->what) {					
		case mouseDown:
			HandleMouseDown(event);
			break;
			
		case keyDown:
		case autoKey:
			HandleKeyPress(event);
			break;
			
		case activateEvt:
			break;
		
		case updateEvt:
			break;
			
		case diskEvt:
			break;
			
		case osEvt:
			break;
		}
	}
	
// Basic handler of keyDown events
void HandleKeyPress (EventRecord *event) {
	char key;
	
	key = event->message & charCodeMask;
	
	if (event->modifiers & cmdKey)
		HandleMenuCommand(MenuKey(key));
	}

// Basic handler of mouseDown events
void HandleMouseDown (EventRecord *event) {
	WindowPtr	theWindow;
	short		part;
	
	part = FindWindow(event->where, &theWindow);
	
	switch (part) {
		case inMenuBar:
			HandleMenuCommand(MenuSelect(event->where));
			break;
			
		case inSysWindow:
			SystemClick(event, theWindow);
			break;
			
		case inContent:
			if (theWindow != FrontWindow())
				SelectWindow(theWindow);
			break;
			
		case inDrag:
			DragWindow(theWindow, event->where, &qd.screenBits.bounds);
			break;
			
		case inGrow:			
			break;
			
		case inGoAway:		
			break;
			
		case inZoomIn:
		case inZoomOut:
			break;
		}
	}

// Simple menu command handler
void HandleMenuCommand (long menuStuff) {
	short theMenu, theItem;
	Str63 theString;
	
	if (! menuStuff) return;
	
	theMenu = (menuStuff & 0xFFFF0000) >> 16;
	theItem = menuStuff & 0x0000FFFF;
	
	switch(theMenu) {
		case mApple:
			switch (theItem) {
				case iAbout:
					(void) NoteAlert(rAbout, NewModalFilterProc(ModalFilter));
					break;
			
				default:
					GetMenuItemText(GetMenuHandle(theMenu), theItem, theString);
					OpenDeskAcc(theString);
					break;
				}
			break;
		
		case mFile:
			switch (theItem) {				
				case iClient:
					XAOLLogin();
					break;
				
				case iSignOff:
					SignOff();
					break;
				
				case iReload:
					StartTest();
					break;
								
				case iQuit:				
					gQuit = true;
					break;
				
				default:
					break;
				}
			break;
		}
	
	HiliteMenu(0);
	DrawMenuBar();
	}

Boolean ModalFilter (DialogPtr theDialog, EventRecord *theEvent, DialogItemIndex *itemHit) {
	Idle();
	return (StdFilterProc(theDialog, theEvent, itemHit));
	}

void XAOLLogin () {
	DialogPtr theDialog;
	DialogItemIndex theItem;
	OSErr err;
	
	theDialog = GetNewDialog(rClient, NULL, (WindowPtr) -1L);
	if (theDialog == NULL) {
		SysBeep(7);
		ExitToShell();
		}
	
	err = SetDialogDefaultItem(theDialog, 1);
	err = SetDialogCancelItem(theDialog, 2);
	ShowWindow(theDialog);
	
	do {
		Str255 theSN, thePW;
		Handle theHandle;
		Rect theRect;
		DialogItemType theType;
			
		ModalDialog(NewModalFilterProc(ModalFilter), &theItem);
		switch (theItem) {
			case 1:				
				GetDialogItem(theDialog, 6, &theType, &theHandle, &theRect);
				GetDialogItemText(theHandle, theSN);
				GetDialogItem(theDialog, 7, &theType, &theHandle, &theRect);
				GetDialogItemText(theHandle, thePW);
				AOLTest(p2cstr(theSN), p2cstr(thePW));
				break;
			
			case 2:
				theItem = 1;
				break;
				
			default:
				break;			
			}
		} while (theItem != 1);
		
	DisposeDialog(theDialog);
	}