/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// ThreadedConnection.c

#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <netdb.h>
#include <stdio.h>

#include "ThreadedConnection.h"

pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
int tcRunning = false;

void 	TCThreadProc 	(void* ptr);
void 	TCThreadRun		(ThreadedConnection* tc);
void 	TCClientWaiting (ThreadedConnection* tc);
void 	TCDataWaiting 	(ThreadedConnection* tc);

void TCInit () {
	pthread_mutex_init(&mutex, NULL);
	tcRunning = true;
	}

void TCShutdown () {
	pthread_mutex_destroy(&mutex);
	tcRunning = false;
	}
	
ThreadedConnection* TCNew () {
	ThreadedConnection* tc;
	
	tc = (ThreadedConnection*) malloc(sizeof(ThreadedConnection));
	if (tc != NULL) {
		tc->socket = -1;
		tc->connected = false;
		tc->checkValue = 31337;
		tc->state = S_Dormant;
		tc->threadOpen = false;
		tc->threadRunning = false;
		tc->dataIn = NULL;
		tc->connectionIn = NULL;
		tc->privateData = NULL;
		}
	
	return (tc);
	}

TC_Error TCOpenThread (ThreadedConnection* tc) {
	if (TCIsThreadOpen(tc) == false) {
		if (pthread_create(&tc->thread, NULL, (void *(*)(void *)) TCThreadProc, (void*) tc) < 0)
			return (TC_ThreadError);
		tc->threadOpen = true;
		tc->threadRunning = false;
		}
	else
		return (TC_ThreadOpen);

	printf("Thread is open...\n");	
	return (TC_NoError);
	}

TC_Error TCCloseThread (ThreadedConnection* tc) {
	if (TCIsThreadOpen(tc) && tc->threadRunning == true) {
		pthread_cancel(tc->thread);
		tc->threadOpen = false;
		tc->threadRunning = false;
		}

	return (TC_NoError);
	}

TC_Error TCJoinThread (ThreadedConnection* tc) {
	if (TCIsThreadOpen(tc) && tc->threadRunning == false) {
		if (pthread_join(tc->thread, NULL))
    		return (TC_ThreadError);
    	}
    else
    	return (TC_ThreadNotOpen);
    
    return (TC_NoError);
	}
	
int TCIsValid (ThreadedConnection* tc) {
	return (tc != NULL && tc->checkValue == 31337);
	}

int TCIsOpen (ThreadedConnection* tc) {
	return (tc->state != S_Dormant);
	}

int TCIsThreadOpen (ThreadedConnection* tc) {
	return (tc->threadOpen);
	}

int TCIsThreadRunning (ThreadedConnection* tc) {
	return (tc->threadRunning);
	}
	
void TCDispose (ThreadedConnection* tc) {
	if (TCIsValid(tc)) {
		TCClose(tc);
		free(tc);
		}
	}

TC_Error TCConnect (ThreadedConnection* tc, char* host, int port) {
	struct sockaddr_in address;
	struct hostent *hostinfo;
	int result;
	
	TCClose(tc);
	
	hostinfo = gethostbyname(host);
	if (! hostinfo)
		return (TC_GetHostError);
		
	tc->socket = socket (AF_INET, SOCK_STREAM, 0);
	if (socket < 0)
		return (TC_SocketError);
	
	address.sin_family = AF_INET;
	address.sin_port = htons(port);
	memcpy(&address.sin_addr.s_addr, hostinfo->h_addr, hostinfo->h_length);
	
	result = connect(tc->socket, (struct sockaddr*) &address, sizeof(address));
	if (result == -1) {
		close(tc->socket);
		return (TC_ConnectError);
		}

	tc->state = S_Active;
	tc->connected = true;
	
	TCOpenThread(tc);
//	printf("Joining the thread\n");  
//	TCJoinThread(tc);
	
	return (TC_NoError);
	}

void TCClose (ThreadedConnection* tc) {
	if (TCIsValid(tc) && TCIsOpen(tc)) {
		close(tc->socket);
		tc->state = S_Dormant;
		tc->connected = false;
		}
	}
	
TC_Error TCSend (ThreadedConnection* tc, char* buffer, int len) {
	int bytesleft = len, written = 0;
	
	TCLock(tc);
	
	printf("TCSend Called\n");
	
	do {
    	if ((written = write(tc->socket, buffer, bytesleft)) <= 0) {
    		TCUnlock(tc);
			return (TC_WriteError);
			}

		bytesleft -= written;
		buffer += written;
  		} while(bytesleft > 0);
	
	TCUnlock(tc);
	
	return (TC_NoError);
	}

TC_Error TCRead (ThreadedConnection* tc, char* buffer, int len, int* numRead) {
	int bytesread = 0;
	int total = 0;

	*numRead = 0;
	TCLock(tc);	

	if ((bytesread = read(tc->socket, buffer, len)) < 0) {
		TCUnlock(tc);
		return (TC_ReadError);
		}
		
	*numRead = bytesread;
 	printf("TCRead - read %i bytes.\n", bytesread);
 	
	TCUnlock(tc);
	
	return (TC_NoError);
	}

int TCLock (ThreadedConnection* tc) {
	return (pthread_mutex_lock(&mutex));
	}

int TCUnlock (ThreadedConnection* tc) {
	return (pthread_mutex_unlock(&mutex));
	}

// Private Code

void TCThreadProc (void* ptr) {
	ThreadedConnection* tc = (ThreadedConnection*) ptr;
	
	if (TCIsValid(tc)) {
		tc->threadRunning = true;
		
		TCThreadRun(tc);
		
		tc->threadRunning = false;
		}
		
	pthread_exit(NULL);
	}

void TCThreadRun(ThreadedConnection* tc) {
	struct timeval timeout;
	fd_set fds;
	int maxfd, x;

	timeout.tv_sec = 10;
	timeout.tv_usec = 0;
	maxfd = tc->socket + 1;

	while (tcRunning && tc->connected == true) {
	//	printf("Polling the socket...\n");

		FD_ZERO(&fds);
		FD_SET(tc->socket, &fds);

		if ((x = select(maxfd, &fds, NULL, NULL, &timeout)) == -1) {
			printf("select failed...\n");
			TCClose(tc);
			return;
			}
		else {
			if (x == 0) {
				// TODO - Timeout
				}
			else {
				printf("Data is waiting...\n");
				if (tc->state == S_Passive)
					TCClientWaiting(tc);
				else
					TCDataWaiting(tc);
				}
  			}
  		}

	printf("Exiting TCThreadRun...\n");
	}

void TCClientWaiting (ThreadedConnection* tc) {
	if (TCIsValid(tc)) {
		if (tc->connectionIn)
			tc->connectionIn(tc);
		}
	}

void TCDataWaiting (ThreadedConnection* tc) {
	printf("Data is read...\n");
	if (TCIsValid(tc)) {
		if (tc->dataIn)
			tc->dataIn((struct ThreadedConnection*) tc);
		}
	}
