/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// ThreadedConnection.h

#ifndef __THREADED_CONNECTION__
#define __THREADED_CONNECTION__

#include <pthread.h>

// Types
enum TC_State {
	S_Active,
	S_Passive,
	S_Dormant
};

typedef enum TC_Error {
	TC_NoError,
	TC_GetHostError,
	TC_ConnectError,
	TC_SocketError,
	TC_WriteError,
	TC_ReadError,
	TC_ThreadError,
	TC_ThreadNotOpen,
	TC_ThreadOpen
} TC_Error;

typedef void (*TCCallback) (struct ThreadedConnection*);

typedef struct ThreadedConnection {
	// Private Data
	int checkValue;
	void* privateData;
	
	// Thread Info
	pthread_t thread;
	int threadOpen;
	int threadRunning;
	
	// Socket
	int socket;
	
	// Connection Information
	int connected;
	int state;
	
	// Callbacks
	TCCallback dataIn;
	TCCallback connectionIn;
} ThreadedConnection;

// Functions
void 				TCInit 				();
void 				TCShutdown		 	();
ThreadedConnection* TCNew	 			();
TC_Error 			TCOpenThread 		(ThreadedConnection* tc);
TC_Error 			TCCloseThread 		(ThreadedConnection* tc);
TC_Error 			TCJoinThread 		(ThreadedConnection* tc);
int 				TCIsValid 			(ThreadedConnection* tc);
int 				TCIsOpen 			(ThreadedConnection* tc);
int 				TCIsThreadOpen 		(ThreadedConnection* tc);
int 				TCIsThreadRunning 	(ThreadedConnection* tc);
void 				TCDispose 			(ThreadedConnection* tc);
TC_Error			TCConnect 			(ThreadedConnection* tc, char* host, int port);
void 				TCClose 			(ThreadedConnection* tc);
TC_Error 			TCSend 				(ThreadedConnection* tc, char* buffer, int len);
TC_Error 			TCRead 				(ThreadedConnection* tc, char* buffer, int len, int* numRead);
int 				TCLock 				(ThreadedConnection* tc);
int 				TCUnlock 			(ThreadedConnection* tc);

#endif __THREADED_CONNECTION__
