/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// MacConnection.c

#include <stdio.h>

#include "DCon.h"
#include "ThreadedConnections.h"
#include "OTTools.h"

#include "AOLSession.h"
#include "P3.h"
#include "ConnectionTypes.h"
#include "Events.h"

size_t myConnectionListSize = 25;					// 25 connection maximum
ThreadedConnectionPtr myConnectionList = NULL;

void ClientReady (ThreadedConnectionPtr connection);
void ClientDisconnect (ThreadedConnectionPtr connection);
void ClientDataRecieved (ThreadedConnectionPtr connection);
void PrintTheData (char* buffer, int len);

ConnectionError mcError = CE_NoError;

XAOLError Mod_ASInit () {
	OSErr err;
	
	//debugstr("ASInit Called!");
	dprintf("\n\n\n\n\n\n\n");
	// Initialize the connections library
	err = InitConnections();
	if (err != noErr)
		// TODO - Fix debug macros
		dprintf("[XAOL] Unable to init connections library! %i\n", err);
		
	// Create a pool of dormant connection records
	myConnectionList = NewConnectionPool(myConnectionListSize, "dormant connection");
	
	return (No_Error);
	}

XAOLError Mod_ASShutdown () {
	int i;
	
	//debugstr("ASShutdown Called!");
	// Kill every connection, shutdown the module
	for (i = 0; i < myConnectionListSize; i++)
		DisposeConnection (&myConnectionList[i]);
				
	ShutdownConnections();
	
	return (No_Error);
	}
	
XAOLError Mod_ASConnect (AOLSession* session, void* connectionData) {
	TCPConnectionData* cd = (TCPConnectionData*) connectionData;
	ThreadedConnectionPtr connection;
	ConnectionError error;
	char hostname[1024];	
	
	//debugstr("Mod_ASConnect Called!");
	connection = GetDormantConnection(myConnectionList, myConnectionListSize);
	if (connection != NULL) {
		session->status = SS_Disconnected;
		session->connectionData = (void*) connection;
		connection->yourData = (Ptr) session;
		connection->ready = ClientReady;
		connection->disconnect = ClientDisconnect;
		connection->dataIn = ClientDataRecieved;
		snprintf(hostname, 1024, "%s:%i", cd->hostname, cd->port);
		error = OpenConnectionActive (connection, hostname);
		if (error != CE_NoError) {
			// TODO - Fix debug macros
			dprintf("[XAOL] Error Opening Connection! %i\n", error);
			mcError = error;
			return (AE_ConnLevelErr);
			}
		}
	else
		return (Mem_Error);
	
	while (session->status != SS_Connected)
		YieldToThread(connection->thread);
	
	XAEDispatch (kConnectEvent, session, NULL);
	return (No_Error);
	}

XAOLError Mod_ASDisconnect (AOLSession* session) {
	ConnectionError error;
	ThreadedConnectionPtr connection;
	
	//debugstr("Mod_ASDisconnect Called!");
	connection = (ThreadedConnectionPtr) session->connectionData;
	
	if (IsConnectionDormant(connection))
		return (AE_StateError);
		
	error = CloseTheConnection(connection);
	XAEDispatch (kDisconnectEvent, session, NULL);	
	if (error != CE_NoError) {
		// TODO - Fix debug macros
		dprintf("[XAOL] Error Closing Connection! %i\n", error);
		mcError = error;
		return (AE_ConnLevelErr);
		}
		
	session->status = SS_Disconnected;
	return (No_Error);
	}
	
XAOLError Mod_ASSend (AOLSession* session, char* buffer, int len) {
	ConnectionError error;
	size_t dataSent;
	
	if (IsConnectionDormant((ThreadedConnectionPtr) session->connectionData)) {
		dprintf("[XAOL] State Error");
		return (AE_StateError);
		}
	
	dprintf("[XAOL] Sending data...\n");
	PrintTheData(buffer, len);
	
	error = SendDataRaw ((ThreadedConnectionPtr) session->connectionData, buffer, len, &dataSent);
	if (error != CE_NoError) {
		// TODO - Fix debug macros
		dprintf("[XAOL] Error Sending Data! (%i) (size/sent: %i/%i\n", error, len, dataSent);
		mcError = error;
		return (AE_ConnLevelErr);
		}
	
	dprintf("[XAOL] Done sending data...\n");
	
	return (No_Error);
	}
	
XAOLError Mod_ASReceive (AOLSession* session, char* buffer, int len, int* read) {
	ThreadedConnectionPtr connection;
	size_t dataWaiting;
	
	connection = (ThreadedConnectionPtr) session->connectionData;
	
	if (IsConnectionDormant(connection))
		return (AE_StateError);
	
	dprintf("[XAOL] Reading data...\n");
	*read = ReadData (connection->ep, buffer, len, &dataWaiting);
	dprintf("[XAOL] Done reading data...\n");
	
	return (No_Error);
	}

void Mod_ASSessionKill (AOLSession* session) {
	#pragma unused (session)
	/* We don't do anything here because we operate off a pool
		however in some other implementations there may well be
		memory that needs to be released.
	*/
	}

int Mod_ASGetError () {
	int result = (int) mcError;
	mcError = CE_NoError;
	return (result);
	}

void Mod_ASIdle () {
	YieldToAnyThread();
	}
	
void ClientReady (ThreadedConnectionPtr connection) {
	AOLSession* session;
	
	session = (AOLSession*) connection->yourData;
	session->status = SS_Connected;
	}
	
void ClientDisconnect (ThreadedConnectionPtr connection) {
	AOLSession* session;
	
	session = (AOLSession*) connection->yourData;
	session->status = SS_Disconnected;
	}
	
void ClientDataRecieved (ThreadedConnectionPtr connection) {
	AOLSession* session;
	P3* p3;
	XAOLError error;
	char* buf = connection->buffer;
	size_t size = connection->bytes;
	int packetCount = 0;
	char tempStr[250];
	int bytesLeft;
	
	if (connection->bytes <= 0) {
		dprintf("Data received of an invalid length\n");
		return;
		}
		
	if (connection->bytes > 0) {
		dprintf("Dumping the packet.\n");
		PrintTheData(connection->buffer, connection->bytes);
		}
		
	session = (AOLSession*) connection->yourData;
	
	do {				
		p3 = P3New (P3_TYPE_INIT);
		if (p3 == NULL) {
			// TODO - Fix debug macros
			dprintf("[XAOL] Error Creating P3! %i\n");
			return;
			}			
			
		packetCount++;
		dprintf("[XAOL] Parsing packet %i.\n", packetCount);
			
//		//debugstr("about to parse it");
		error = P3ParsePacket (p3, buf, size, &bytesLeft);
		sprintf(tempStr, "There are %i bytes left to be processed.\n", bytesLeft);
//		//debugstr(tempStr);
		if (error != No_Error) {
			// TODO - Fix debug macros
			dprintf("[XAOL] Error Parsing P3! %i\n", error);
			return;
			}
		else {			
			if (p3->length - 3 > 0) {
				dprintf("Dumping the P3 data packet.\n");
				PrintTheData(p3->data, p3->length - 3);
				}
//			//debugstr("about to recieve it");
			error = ASReceiveP3	(session, p3);
			if (error != No_Error) {
				// TODO - Fix debug macros
				dprintf("[XAOL] Error Recieving P3! %i\n", error);
				return;
				}
			}
			
		buf += size - bytesLeft;
		size = bytesLeft;
		
		// The size is what let's left after taking away from the buffer
		P3Dispose(p3);
		} while (bytesLeft > 0);
	
	connection->bytes = 0;
	}
	
void PrintTheData (char* buffer, int len) {
	int i;
	
	dprintf("** Begin Data Dump **\n");
	for (i = 0; i < len; i+=2) {
		dprintf("%02x", buffer[i]);
		if (i + 1 < len)
			dprintf(" %02x\t", buffer[i + 1]);
		}
	dprintf("\n** End Data Dump **\n");
	}