/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// MacModules.c

#include <stdlib.h>
#include <string.h>

#include <CodeFragments.h>
#include <Files.h>
#include <TextUtils.h>
#include <Errors.h>

#include "Modules.h"

#define kXAOLModuleName	"\pxaol module"

typedef struct MacPrivateMData {
	CFragConnectionID id;
	Ptr main;
} MacPrivateMData;

OSErr modErr = noErr;

/* TODO - Write some Unix/DOS style path parsing routines
		for right now assume the current directory
*/
typedef void (*MainProcPtr)(void);
XAOLError MPrivateLoad (Module* module, char* path) {
	FSSpec fs;
	OSErr err;
	MacPrivateMData* mdd;
	Str255 error;
	
	if (! strlen(path))
		return (Arg_Error);
	
	if (! MIsValid(module))
		return (ME_InvalidModule);
	
	mdd = (MacPrivateMData*) MAlloc (sizeof(MacPrivateMData));
	if (mdd == NULL) {
		XAOL_DEBUG_Print_("Error making module ref\n");
		return (Mem_Error);
		}
	
	err = FSMakeFSSpec(0, 0, c2pstr(path), &fs);
	if (err != noErr) {
		XAOL_DEBUG_Print_("Error making FSSpec\n");
		return (File_Error);
		}
		
	err = GetDiskFragment(&fs, 0, kCFragGoesToEOF, "\p", kPrivateCFragCopy, &mdd->id, &mdd->main, error);
	// TODO - Something funny is going on here
	XAOL_DEBUG_Printf_("The error string from GetDiskFragment() is : %s\n", p2cstr(error));
	
	if (err != noErr) {
		modErr = err;
		return (ME_InternalError);
		}	
	
	module->data = (void*) mdd;
	
	return (No_Error);
	}
	
XAOLError MPrivateUnload (Module* module) {
	MacPrivateMData* mdd;
	OSErr err;
	CFragConnectionID id;
	
	if (! MIsValid(module))
		return (ME_InvalidModule);
	
	mdd = (MacPrivateMData*) module->data;
	
	id = mdd->id;
	MFree((void*) module->data);
	module->data = NULL;
	
	err = CloseConnection(&id);
	if (err != noErr) {
		modErr = err;
		return (ME_InternalError);
		}
	
	return (No_Error);
	}
	
XAOLError MLoadDirectory (char* directory) {
	#pragma unused(directory)
	return (No_Error);
	}
	
XAOLError MSendMessage (Module* module, int messageType, void* messageData) {
	MacPrivateMData* mdd;
	
	if (! MIsValid(module))
		return (ME_InvalidModule);
	
	mdd = (MacPrivateMData*) module->data;
	((void (*) (int, void*)) mdd->main) (messageType, messageData);
	return (No_Error);
	}

int MGetError () {
	int error = (int) modErr;
	
	modErr = noErr;
	
	return (error);
	}

XAOLError MGetSymbol (Module* module, char* symbol, void** symAddr) {
	OSErr err;
	MacPrivateMData* mdd;
	CFragSymbolClass symClass;
	Str255 es;
	
	XAOL_DEBUG_Printf_("Looking up symbol %s\n", symbol);
	
	if (! MIsValid(module))
		return (ME_InvalidModule);
	
	mdd = (MacPrivateMData*) module->data;
	err = FindSymbol (mdd->id, c2pstr(symbol), (Ptr*) symAddr, &symClass);
	if (err != noErr) {
		NumToString((long) err, es);
		XAOL_DEBUG_Printf_("Error returned by FindSymbol() : %s\n", p2cstr(es));
		modErr = err;
		return (ME_SymbolNotFound);
		}
		
	return (No_Error);
	}