/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// UnixConnection.c

#include <stdio.h>

#include "XErrors.h"
#include "UnixErrors.h"
#include "ConnectionTypes.h"
#include "AOLSession.h"
#include "Events.h"

#include "ThreadedConnection.h"

#ifdef AS_MODULE
#define ASInit Mod_ASInit
#define ASShutdown Mod_ASShutdown
#define ASConnect Mod_ASConnect
#define ASDisconnect Mod_ASDisconnect
#define ASSend Mod_ASSend
#define ASReceive Mod_ASReceive
#define ASSessionKill Mod_ASSessionKill
#define ASGetError Mod_ASGetError
#define ASIdle Mod_ASIdle
#endif

UnixError error;

void PrintData (char* buffer, int len);
void PrivateDataIn (struct ThreadedConnection* tc);

XAOLError ASInit () {
	printf("Initializing Threaded Connection Layer...\n");
	TCInit();
	error = UE_NoError;
	return (No_Error);
	}
	
XAOLError ASShutdown () {
	TCShutdown();
	return (No_Error);
	}
	
XAOLError ASConnect (AOLSession* session, void* connectionData) {
	TCPConnectionData* tcpData;
	ThreadedConnection* tc = NULL;
	TC_Error err;
	
	tcpData = (TCPConnectionData*) connectionData;
	if (tcpData == NULL || strlen(tcpData->hostname) == 0 || tcpData->port < 0)		
		return (Arg_Error);
	
	printf("Connecting to %s:%i\n", tcpData->hostname, tcpData->port);
	
	tc = TCNew();
	if (tc != NULL) {
		err = TCConnect(tc, tcpData->hostname, tcpData->port);
		if (err != TC_NoError) {
			printf("An error occured opening the connection...\n");
			error = (int) err;
			return (AE_ConnLevelErr);
			}
		
		printf("Connection opened successfully.\n");
		session->connectionData = (void*) tc;
		session->status = SS_Connected;
		tc->dataIn = PrivateDataIn;
		tc->privateData = (void*) session;
		}
	else
		return (Mem_Error);
	
	XAEDispatch (kConnectEvent, session, NULL);
	return (No_Error);
	}
	
XAOLError ASDisconnect (AOLSession* session) {
	ThreadedConnection* tc = (ThreadedConnection*) session->connectionData;
	
	TCClose(tc);
	XAEDispatch (kDisconnectEvent, session, NULL);
	
	return (No_Error);
	}
	
XAOLError ASSend (AOLSession* session, char* buffer, int len) {
	ThreadedConnection* tc = (ThreadedConnection*) session->connectionData;
	TC_Error err;

	//printf("ASSend Called!\n");
	
	PrintData(buffer, len);

	err = TCSend (tc, buffer, (int) len);
	if (err != TC_NoError) {
		fprintf(stderr, "Error returned by TCSend(): %i\n", err);
		error = (int) err;
		return (AE_ConnLevelErr);
		}
	else
		perror("ASSend Successful!\n");

	return (No_Error);
	}
	
XAOLError ASReceive (AOLSession* session, char* buffer, int len, int* read) {
	ThreadedConnection* tc = (ThreadedConnection*) session->connectionData;
	TC_Error err;
	
	//printf("Entering ASRecieve\n");

	err = TCRead (tc, buffer, (int) len, (int*) read);
	if (err != TC_NoError) {
		error = (int) err;
		return (AE_ConnLevelErr);
		}
	
	PrintData(buffer, *read);

	return (No_Error);
	}
	
void ASSessionKill (AOLSession* session) {
	}
	
int ASGetError () {
	int lerror = (int) error;
	
	error = UE_NoError;
	return (lerror);
	}

void ASIdle () {
	}

void PrivateDataIn (struct ThreadedConnection* tc) {
	AOLSession* session;
	P3* p3;
	TC_Error error;
	XAOLError xerror;
	char buffer[4096];
	int numRead;
	char* buf;
	int size;
	int numLeft;

	//printf("Entered into private data handling routine...\n");
	session = (AOLSession*) tc->privateData;

	error = TCRead(tc, buffer, 4096, &numRead);
        if (error != TC_NoError || numRead <= 0) {
		perror("Error reading data!\n");
		return;
                }

	PrintData(buffer, numRead);
	//printf("Phase 2\n");
	buf = buffer;
	size = numRead;

	if (session != NULL) {
		do {
			//printf("1");
			p3 = P3New(P3_TYPE_INIT);
			if (p3 == NULL) {
				perror("Error creating P3!\n");
				return;
				}

			//printf("2");
			xerror = P3ParsePacket(p3, buf, size, &numLeft);
			if (xerror != No_Error) {
				perror("Error parsing packet!\n");
				P3Dispose(p3);
				return;
				}
			
			//printf("Handing it off to be processed.\n");
			xerror = ASReceiveP3(session, p3);
			if (error != No_Error) {
				perror("Error handling p3\n");
				P3Dispose(p3);
				return;
				}
			
			buf += numLeft;
			size = numLeft;

			P3Dispose(p3);
			} while (numLeft > 0);
		}
	}

void PrintData (char* buffer, int len) {
	int i;
	
	perror("*** Begin Data Dump ***\n");

	for (i = 0; i < len; i+=2) {
		fprintf(stderr, "%02x", buffer[i]);
		if (i + 1 < len)
			fprintf(stderr, " %02x\t", buffer[i+1]);
		}

	perror("\n*** End Data Dump***\n");	
	}