/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// WinConnection.c

#include <stdio.h>

#include "XErrors.h"
#include "UnixErrors.h"
#include "ConnectionTypes.h"
#include "AOLSession.h"

#include "ThreadedConnection.h"

UnixError error;

void PrivateDataIn (struct ThreadedConnection* tc);

XAOLError ASInit () {
	printf("Initializing TC...\n");
	TCInit();
	error = UE_NoError;
	return (No_Error);
	}
	
XAOLError ASShutdown () {
	TCShutdown();
	return (No_Error);
	}
	
XAOLError ASConnect (AOLSession* session, void* connectionData) {
	TCPConnectionData* tcpData;
	ThreadedConnection* tc = NULL;
	TC_Error err;
	
	tcpData = (TCPConnectionData*) connectionData;
	if (tcpData == NULL || strlen(tcpData->hostname) == 0 || tcpData->port < 0)		
		return (Arg_Error);
	
	printf("Connecting to %s:%i\n", tcpData->hostname, tcpData->port);
	
	tc = TCNew();
	if (tc != NULL) {
		err = TCConnect(tc, tcpData->hostname, tcpData->port);
		if (err != TC_NoError) {
			printf("An error occured opening the connection...\n");
			error = (int) err;
			return (AS_ConnLevelErr);
			}
		
		printf("Connection opened successfully.\n");
		session->connectionData = (void*) tc;
		session->status = SS_Connected;
		tc->dataIn = PrivateDataIn;
		}
	else
		return (Mem_Error);
	
	return (No_Error);
	}
	
XAOLError ASDisconnect (AOLSession* session) {
	ThreadedConnection* tc = (ThreadedConnection*) session->connectionData;
	
	TCClose(tc);
	
	return (No_Error);
	}
	
XAOLError ASSend (AOLSession* session, char* buffer, int len) {
	ThreadedConnection* tc = (ThreadedConnection*) session->connectionData;
	TC_Error err;

	printf("ASSend Called!\n");
	
	err = TCSend (tc, buffer, (int) len);
	if (err != TC_NoError) {
		printf("Error returned by TCSend(): %i\n", err);
		error = (int) err;
		return (AS_ConnLevelErr);
		}
	else
		printf("ASSend Successful!\n");

	return (No_Error);
	}
	
XAOLError ASRecieve (AOLSession* session, char* buffer, int len, int* read) {
	ThreadedConnection* tc = (ThreadedConnection*) session->connectionData;
	TC_Error err;
	
	err = TCRead (tc, buffer, (int) len, (int*) read);
	if (err != TC_NoError) {
		error = (int) err;
		return (AS_ConnLevelErr);
		}
		
	return (No_Error);
	}
	
void ASSessionKill (AOLSession* session) {
	}
	
int ASGetError () {
	int lerror = (int) error;
	
	error = UE_NoError;
	return (lerror);
	}

void ASIdle () {
	}

void PrivateDataIn (struct ThreadedConnection* tc) {
	AOLSession* session;
	P3* p3;
	TC_Error error;
	XAOLError xerror;
	char buffer[4096];
	int numRead;
	char* buf;
	int size;
	int numLeft;

	session = (AOLSession*) tc->privateData;

	error = TCRead(tc, buffer, 4096, &numRead);
        if (error != TC_NoError || numRead <= 0) {
		perror("Error reading data!\n");
		return;
                }
	
	buf = buffer;
	size = numRead;

	if (session != NULL) {
		do {
			p3 = P3New(P3_TYPE_INIT);
			if (p3 == NULL) {
				perror("Error creating P3!\n");
				return;
				}

			xerror = P3ParsePacket(p3, buf, size, &numLeft);
			if (xerror != No_Error) {
				perror("Error parsing packet!\n");
				P3Dispose(p3);
				return;
				}
			
			xerror = ASReceiveP3(session, p3);
			if (error != No_Error) {
				perror("Error handling p3\n");
				P3Dispose(p3);
				return;
				}
			
			buf += numLeft;
			size = numLeft;

			P3Dispose(p3);
			} while (numLeft > 0);
		}
	}
