/*
 	File:		PortBounce.c
 
 	Contains:	Connection bouncing via ThreadedConnections
 
 	Version:	0.1
 
 	Copyright:	 1999 by Blake Watters, all rights reserved.
 				
 				Permission is granted to use this code freely as long as
 				this notice is preserved.
 
 	Bugs?:		Please send bugs to Blake Watters <h20@midsouth.rr.com>
 
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <SIOUX.h>

#include "ThreadedConnections.h"

typedef struct Configuration {
	char bounceStr[256];
	int port;
	} Configuration;
	
size_t myConnectionListSize = 25;					// 25 connection maximum
ThreadedConnectionPtr myConnectionList = NULL;
Configuration theConfig;

Boolean LoadConfiguration ();
Boolean ConsoleConfig ();
void ParseLine (char* buffer, short pos, short nextCR);
short FindCR (char* buffer, int pos);
void DoBounce ();
void ServerConnect (ThreadedConnectionPtr theConnection);
void ClientDataIn (ThreadedConnectionPtr theConnection);
void ClientDisconnect (ThreadedConnectionPtr theConnection);

void main () {
	OSErr err;
	EventRecord event;
	Boolean gotEvent;
	char key;
	int i;
	
	printf("Port Bouncer - Connection redirecting via ThreadedConnections initializing...\n");
	
	theConfig.bounceStr[0] = 0;
	theConfig.port = -1;
	
	err = InitConnections();
	if (err != noErr)
		printf("Error initializing ThreadedConnections. (Error code %i)\n", err);
	else {
		myConnectionList = NewConnectionPool(myConnectionListSize, "dormant connection");
		if (myConnectionList != NULL) {
			if (LoadConfiguration()) {
				DoBounce();
				
				for(;;) {
					gotEvent = WaitNextEvent(everyEvent, &event, 400, nil);
					if (gotEvent) {
						if (event.what == keyDown || event.what == autoKey)
							key = event.message & charCodeMask;
							if ((event.modifiers & cmdKey) && (key == 'q' || key == 'Q'))
								break;
						}
						
					SIOUXHandleOneEvent(NULL);
					YieldToAnyThread();
					}
				
				printf("PortBounce is shutting down...\n");
				for (i = 0; i < myConnectionListSize; i++)
					DisposeConnection (&myConnectionList[i]);
				ShutdownConnections();
				}
			}
		}
	}

Boolean LoadConfiguration () {
	char name[32];
	OSErr err;
	short file;
	long len;
	char* buffer;
	int pos, nextCR;
	FInfo fi;
	
	return (ConsoleConfig());
	
	do {
		printf("Enter the name of the bounce configuration file [leave blank for bounce.cfg]: ");
		gets(name);
	
		err = GetFInfo(c2pstr(name), 0, &fi);
		if (err != noErr)
			printf("You specified an invalid filename.\n\n");
		} while (err != noErr);
	
	printf("Filename %s : %p \n", name, c2pstr(name));
	
	err = FSOpen(c2pstr(name), 0, &file);
	if (err != noErr) {
		printf("A file system error occured while opening the file. (Error code %i)\n", err);
		return (false);
		}
	
	err = GetEOF(file, &len);
	if (err != noErr) {
		printf("A file system error occured while getting file length. (Error code %i)\n", err);
		return (false);
		}
	
	err = SetFPos(file, fsFromStart, 0);
	if (err != noErr) {
		printf("A file system error occured setting the position. (Error code %i)\n", err);
		return (false);
		}
		
	buffer = NewPtr(len + 1);
	if (buffer == NULL) {
		printf("Error allocating memory for configuration buffer.\n");
		return (false);
		}
	
	err = FSRead(file, &len, buffer);
	if (err != noErr) {
		printf("Error reading configuration data. (Error code %i)\n", err);
		DisposePtr(buffer);
		return (false);
		}
	
	// Parse the file
	pos = 0;
	nextCR = 0;
	do {
		nextCR = FindCR(buffer, pos);
		if (nextCR != -1)
			ParseLine(buffer, pos, nextCR);
		pos = nextCR++;
		} while(nextCR != -1);
	
	DisposePtr(buffer);
	FSClose(file);
	
	// Test the configuration
	if (strlen(theConfig.bounceStr) > 0 && theConfig.port != -1)
		return (true);
	else
		return (false);
	}

Boolean ConsoleConfig () {
	int w;
	
	printf("Enter it bitch: ");
	scanf("%i", &w);
	printf("The number is %i\n", w | 128);
	
	printf("Enter the port to listen on: ");
	scanf("%i", &theConfig.port);
	printf("Enter the site to bounce to (format is host:port): ");
	scanf("%s", &theConfig.bounceStr);
	printf("Bouncing from 127.0.0.1:%i to %s\n", theConfig.port, theConfig.bounceStr);
	return (true);
	}
	
void ParseLine (char* buffer, short pos, short nextCR) {
	int i, x;
	char line[256];
	
	line[0] = NULL;
	
	for (i = pos, x = 0; i < nextCR; i++, x++) {
		if (buffer[i] == '#')
			break;	// Ignore comments
		line[x] = buffer[i];
		}
		
	line[x + 1] = NULL;
	
	if (! strlen(line))
		return;
		
	// Try to read the port first
	if (theConfig.port == -1) {
		printf("Converting the port (%s)\n", line);
		theConfig.port = atoi(line);
		return;
		}
	else if (theConfig.bounceStr[0] == 0) {
		// Copy it the lazy way
		theConfig.bounceStr[0] = x - 1;
		for (i = 0; i < (x - 1); i++)
			theConfig.bounceStr[i + 1] = line[i];
		printf("Converting the connect str (%p)\n", theConfig.bounceStr);
		}
	}
	
short FindCR (char* buffer, int pos) {
	int i;
	
	for (i = pos; buffer[i] != 0x0D && buffer[i] != NULL; i++)
		;
	
	if (buffer[i] == 0x0D)
		return i;
	else
		return -1;
	}
	
void DoBounce () {
	ThreadedConnectionPtr connection = NULL;
	ConnectionError theError;
	
	connection = GetDormantConnection(myConnectionList, myConnectionListSize);
	if (connection) {
		// Setup the callbacks
		connection->listen = ServerConnect;
		connection->dataIn = ClientDataIn;		
		connection->disconnect = NULL;		
		connection->ready = NULL;
		connection->yourData = NULL;
		
		// Open up the server
		theError = OpenConnectionPassive(connection, theConfig.port, 25);
		if (theError == CE_NoError)
			YieldToThread(connection->thread);
		else {
			printf("TC Error #%i\n", theError);
			return; // TODO - Error handling
			}
		}	
	}

void ServerConnect (ThreadedConnectionPtr theConnection) {
	ThreadedConnectionPtr connection;
	ThreadedConnectionPtr theBounce;
	ConnectionError theError;	
	
	connection = GetDormantConnection(myConnectionList, myConnectionListSize);
	if (connection) {
		printf("Connect recieved on local port %i, redirecting...\n", theConfig.port);
		
		connection->dataIn = ClientDataIn;
		connection->disconnect = ClientDisconnect;
		theError = OpenConnectionAcceptor (connection);
		if (theError == CE_NoError) {
			// Open the bounce
			theBounce = GetDormantConnection(myConnectionList, myConnectionListSize);
			if (theBounce != NULL) {
				theBounce->dataIn = ClientDataIn;
				theBounce->disconnect = ClientDisconnect;
				theError = OpenConnectionActive(theBounce, theConfig.bounceStr);
				if (theError != CE_NoError) {
					printf("Error opening the bounce (Error code %i)\n", theError);
					theError = DisconnectClient (theConnection);
					}
				}
			
			// Link the connections
			connection->yourData = (Ptr) theBounce;
			theBounce->yourData = (Ptr) connection;
			
			theError = AcceptConnection (theConnection, connection);			
			}
		else
			theError = DisconnectClient (theConnection);
		}
	}

void ClientDataIn (ThreadedConnectionPtr theConnection) {
	ConnectionError err;
	ThreadedConnectionPtr connection;
	
	connection = (ThreadedConnectionPtr) theConnection->yourData;
	printf("Reading data\n");
	
	if (! IsConnectionValid(connection))
		return;
		
	err = SendDataRaw(connection, theConnection->buffer, theConnection->bytes, NULL);
	if (err != CE_NoError)
		printf("Error in ClientDataIn (%i)\n", err);
	theConnection->bytes = 0;
	}

void ClientDisconnect (ThreadedConnectionPtr theConnection) {
	ConnectionError err;
	ThreadedConnectionPtr connection;
	
	connection = (ThreadedConnectionPtr) theConnection->yourData;
	if (! IsConnectionValid(connection))
		return;
		
	connection = (ThreadedConnectionPtr) theConnection->yourData;
	err = CloseTheConnection(connection);
	if (err != CE_NoError)
		printf("Error in ClientDataIn (%i)\n", err);
	}